var Event = require('@spark-engine/event')

function navigateMenu(event) {
  var link,
      activeEl = document.activeElement

  if(Event.key.isPressed('down')) {
    if (activeEl.tagName.match(/select|button/i) || activeEl.closest('[aria-hidden="true"]')) { return }
    // Prevent page scrolling as a result of keypress
    event.preventDefault()
    event.stopImmediatePropagation()
    link = getNextLink()

  } else if(Event.key.isPressed('up')) {
    // Prevent page scrolling as a result of keypress
    if (activeEl.tagName.match(/select|button/i)) { return }
    event.preventDefault()
    event.stopImmediatePropagation()
    link = getPreviousLink()
  } else {
    link = false
  }

  // Only focus if there is a next or previous link to move to
  // At top or bottom of a list there is no "next" or "previous"
  if (link) link.focus()

}

// Find the next focusable link
function getNextLink() {
  var items = links(),
      pos = items.indexOf(document.activeElement)

  // No active element: return first element
  if (pos == -1) return items[ 0 ]

  // Last element focused, return nothing (no next element)
  else if (pos == items.length) return

  // return next focusable link
  else return items[ pos + 1 ]
}

// Find the previous focusable link
function getPreviousLink (el) {
  var items = links(),
      pos = items.indexOf(document.activeElement)

  // No previous element to select
  if (pos == -1 || pos == 0) return

  // return previous focusable link
  else return items[ pos - 1 ]
}

// Return all links in current modal
function links() {
  var menu = document.activeElement.closest('[role="menu"]')
  return Array.prototype.filter.call(menu.querySelectorAll('a'), function(a) {
    return a.offsetParent != null
  })
}

function setup () {
  Event.ready(function() {
    Event.on(document, 'keydown', 'nav[role="menu"]', navigateMenu, { useCapture: true })
  })
}

module.exports = {
  setup: setup
}
