var toolbox  = require('@spark-engine/toolbox'),
    Event    = require('@spark-engine/event'),
    modal    = require('./_modal'),
    stack    = require('./_stack'),
    searchStack

function autoClose(event) {
  if (!document.activeElement.closest('#search_modal')) {
    modal.get('#search-results').close()
  }
}

function getSearchStack() {
  if (!searchStack) searchStack = stack.get('#search-stack')
  return searchStack
}

// Scroll all navs back to top
function resetScroll () {
  toolbox.each(getSearchStack().root.querySelectorAll('nav'), function (nav) {
    nav.scrollTop = 0
  })
}

function keyWatcher (event) {
  var focusEl = document.activeElement,
      query = document.querySelector('[name=q]')

  // If a letter, number, or backspace is pressed, focus on search input
  if ((event.key.length === 1 || Event.key.isPressed("backspace")) && focusEl != query && !focusEl.tagName.match(/select|button/i)) {
    query.focus()

  // Restore default search input behavior
  // When foucsed, esc clears input
  } else if (focusEl == query && Event.key.isPressed("esc")) {
    query.value = ''
    Event.fire(query, 'input')
  } else if (Event.key.isPressed("down")) {
    if (document.activeElement && document.activeElement.tagName.match(/select|button/i)) { return }
    var result = getSearchStack().currentPanel().querySelector('[role=menu] a')
    if (result) {
      event.preventDefault()
      result.focus()
    }
  }
}

// 1. Focus on query input when characters are typed
// 2. Clear input when escape is pressed
// Show first panel stack when query is cleared
function queryInput (event) {
  var query = event.currentTarget

  if (query.value == '') {
    var searchPanel = document.querySelector('.search-panel')

    getSearchStack().show(0)
    searchPanel.classList.remove('search-in-progress')
    resetScroll()
  }
}


function init() {
  var query       = document.querySelector('[name=q]'),
      searchPanel = document.querySelector('.search-panel'),
      searchModal = modal.get('#search-results')

  if (!searchModal || !query) return

  searchModal.on('open', function() {
    Event.on(document, 'keydown', keyWatcher)
    Event.on(document, 'blur', autoClose)
  })

  // Reset stack when modal is closed and query is clear
  searchModal.on('close', function() {
    // Restore state of search panel
    if (query.value == '' && getSearchStack()) {
      getSearchStack().reset()
      resetScroll()
    }

    searchPanel.classList.remove('search-in-progress')

    Event.off(document, 'keydown', keyWatcher)
    Event.off(document, 'blur', autoClose)
  })
}

function focusQuery(event) {
  var query = document.querySelector('[name=q]')
  if (query && !event.target.closest('[contenteditable=true]')) {
    event.preventDefault()
    query.focus()
  }
}

function setup() {
  Event.change(init)

  Event.ready(function() {
    Event.on('[name=q]', 'input', queryInput)

    // Focus on search when `/` key is pressed
    Event.keyOn('/', focusQuery)
  })

  Event.beforeRender(teardown)
}

function teardown() {
  searchStack = null
}

// Reset search results panel when query is cleared
module.exports = {
  setup: setup
}
