var Event = require('@spark-engine/event'),
    sidebar,
    header,
    footer

// Fixes margins and widths of sidebar elements.
// 
// If there is a header or footer, the sidebar panel needs top and bottom margin
// to ensure the full height navigation scrolls to reveal content
//
// If the scrolling sidebar panel has an scrollbar with width, this sets the width
// of the header and footer to match the panel, showing the full height scrollbar.
//
function fixSizes() {
  if(!sidebar) { return }

  if (header) {
    header.style.width = sidebar.clientWidth + 'px';
    sidebar.style.marginTop = header.clientHeight + 'px';
  }
  if (footer) {
    footer.style.width = sidebar.clientWidth + 'px';
    sidebar.style.marginBottom = footer.clientHeight + 'px';
  }
}

// Save current sidebar scroll position
function saveScrollPosition() {
  if(!sidebar) { return }
  sessionStorage.setItem('spark.sidebar.position', sidebar.parentElement.scrollTop)
}

// Restore previous sidebar scroll position
function restoreScrollPosition() {
  if(sidebar) {
    sidebar.parentElement.scrollTop = sessionStorage.getItem('spark.sidebar.position') || 0
  } else {
    sessionStorage.setItem('spark.sidebar.position', 0)
  }
}

// Triggered during initial page load or subsequent page changes (i.e. Turbolinks)
Event.change(function(){
  sidebar = document.querySelector('#sidebar .spark-nav-sidebar-main')
  header = document.querySelector('#sidebar .spark-nav-sidebar-header')
  footer = document.querySelector('#sidebar .spark-nav-sidebar-footer')

  fixSizes()
  restoreScrollPosition()
})

Event.ready(function() {
  Event.on(document, 'transitionend', '#sidebar .spark-nav-sidebar-main', fixSizes)
  Event.resize.end(fixSizes)
  Event.beforeChange(saveScrollPosition)
})
