var Event   = require('@spark-engine/event'),
    toolbox = require('@spark-engine/toolbox')

function sizeGrid(grid) {
  var min = gridSpan(grid)

  // baseSpan preserves the initial column span setting, this prevents
  // grids from having more columns than specified.
  toolbox.each(gridColumns(grid), function(item) {
    var baseSpan = item.dataset.baseSpan
    item.dataset.span = (min < baseSpan) ? baseSpan : min
  })
}

function breakGrid(grid) {
  if (grid.clientWidth < Number(grid.dataset.gridMin)) {
    grid.firstElementChild.classList.add('break')
  } else {
    grid.firstElementChild.classList.remove('break')
  }
}

// Returns a column span size which fits the greates number of columns
// With each column being larger than the grid's min column width
function gridSpan(grid) {
  var min       = Number(grid.dataset.columnMin),
      gridWidth = grid.clientWidth,
      columns   = gridColumns(grid),
      gutters   = gutterSize(columns[0]),
      trueMin   = min + gutters,
      fits      = Math.floor(gridWidth / trueMin),
      shouldFit = columns.length <= fits,
      columnSize = gridWidth / 12

  // `fits` is how many min-sized colums will fit at current grid width
  // If columns are autosized, and all fit, reset
  if (shouldFit && autoColumns(columns)) { return 0 }
  // If columns should fit, but don't
  else if (12 <= fits) return 1
  else if (6 <= fits) return 2
  else if (4 <= fits) return 3
  else if (3 == fits) return 4
  else if (2 == fits) return 6
  // If there is only one column, graduate the width expansion
  else if (columns.length == 1 && (trueMin < columnSize * 8)) return 8
  else if (columns.length == 1 && (trueMin < columnSize * 10)) return 10
  else return 12
}

function gridColumns(grid) {
  return grid.firstElementChild.children
}

function autoColumns(columns) {
  return Array.prototype.filter.call(columns, function(col) {
    return Number(col.dataset.baseSpan) == 0
  }).length == columns.length
}

function gutterSize(item) {
  var left = window.getComputedStyle(item).marginLeft.replace('px',''),
      right = window.getComputedStyle(item).marginRight.replace('px','')
  return  Number(left) + Number(right)
}

function sizeGrids() {
  toolbox.each(document.querySelectorAll('.spark-grid[data-column-min]'), sizeGrid)
  toolbox.each(document.querySelectorAll('.spark-grid[data-grid-min]'), breakGrid)
}

function init() {
  items = document.querySelectorAll('.spark-grid-column:not([data-base-span])')
  toolbox.each(items, function(item) {
    item.dataset.baseSpan = item.dataset.span || 0
  })
  sizeGrids()
}

function setup(){
  Event.change(init)
  Event.resize(sizeGrids)
}


module.exports = {
  setup: setup,
  gridSpan: gridSpan
}
