var Event    = require('@spark-engine/event'),
    toolbox  = require('@spark-engine/toolbox')

function toggle(event) {
  var trigger = event.currentTarget
  trigger.classList.toggle('active')
  toolbox.each(document.querySelectorAll(trigger.dataset.navToggle), function(nav) {
    nav.classList.toggle('nav-active')
  })

  // If we've triggered a nav toggle (which hasn't been made active yet),
  // set keyboard listener scope to watch for escape and close open menus
  if (document.querySelector('[data-nav-toggle].active')) {
    Event.key.setScope('nav-toggle')

  // If there are no other toggles active, restore keyboard listener scope
  } else {
    Event.key.setScope('all')
  }
}

function toggleActive(event) {
  var item = event.currentTarget,
      nav = item.closest('nav.nav-active'),
      button = nav.querySelector('[data-nav-toggle]') || document.querySelector(nav.dataset.toggledBy)

  if (button) { Event.fire(button, 'click') }
}

function esc() {
  toolbox.each(document.querySelectorAll('[data-nav-toggle].active'), function(button) {
    Event.fire(button, 'click')
  })
}

function initToggleActive() {
  var buttons = document.querySelectorAll('nav [data-nav-toggle]')
  toolbox.each(buttons, function(button) {
    toolbox.each(button.closest('nav').querySelectorAll('.nav-item:not([data-set-text])'), function(item) {
      var text = item.querySelector('.nav-item-text').textContent
      item.dataset.setText = '#' + button.id + ' .nav-toggle-text:' + text

      if (item.matches('.active')) { 
        button.querySelector('.nav-toggle-text').innerText = text 
      }
    })
  })
}

function setup() {
  Event.ready(function(){
    Event.on(document, 'click', '[data-nav-toggle]', toggle)
    Event.on(document, 'click', 'nav.nav-active .nav-item:not([href="#"])', toggleActive)
    Event.keyOn('esc', 'nav-toggle', esc)
  })

  Event.change(initToggleActive)
}

module.exports = { setup: setup }
