var Event = require('@spark-engine/event'),
    activeEl

// Copy to clipboard handler
function copyClick(event) {
  var trigger = event.currentTarget,
      el = document.querySelector(trigger.dataset.copy)

  // Prevent default behavior
  event.preventDefault()

  copy(el, trigger)
}

function copy(el, trigger) {
  // Is the element selectable?
  if (el) {

    var textArea = document.createElement("textarea")

    textArea.value = el.textContent
    document.body.appendChild(textArea)
    textArea.focus()
    textArea.select()

    try {

      document.execCommand('copy')

      if (trigger) {
        // Trigger copied state
        trigger.classList.add('copied')

        // Remove copied state after a couple seconds
        Event.afterAnimation(trigger, function() {
          trigger.classList.remove('copied')
        }, 1500)
      }

    // If it breaks, go nuclear
    } catch (err) {
      console.error('copy failed')
    }

    // a119: return focus to previous focused element
    textArea.blur()
    if (activeEl) activeEl.focus()

    document.body.removeChild(textArea)
  }
}

// Public API function
var setup = function() {
  // Listen for click events
  Event.on(document, 'click', '[data-copy]', copyClick)

  // a119: track focused element to restore focus later
  Event.on(document, 'mousedown keydown', '[data-copy]', function(){
    activeEl = document.activeElement
  })
}

module.exports = {
  setup: setup,
  copy: copy
}
