var Event = require('@spark-engine/event'),
    esvg = require('../../_esvg')

function createContainer() {
  var el = document.createElement('div')

  el.classList.add('spark-toast-container')

  document.body.appendChild(el)

  return el
}

function createToastButton() {
  var el = document.createElement('button')

  el.setAttribute('type', 'button')
  el.setAttribute('class', 'spark-toast-close')
  el.setAttribute('data-dismiss', 'toast')
  el.setAttribute('aria-label', 'Close')
  el.insertAdjacentElement('beforeend', esvg.use('close'))

  return el
}

function createToast(message, theme) {
  var el = document.createElement('div')
  var toastMessage = document.createElement('div')
  var button = createToastButton()

  el.classList.add('spark-toast')
  el.classList.add('theme-' + theme)
  el.setAttribute('role', 'alert')
  toastMessage.classList.add('spark-toast-message')
  toastMessage.innerHTML = message
  el.insertAdjacentElement('afterbegin', toastMessage)
  el.insertAdjacentElement('beforeend', button)

  if (theme == 'danger') {
    var icon = esvg.use('alert-triangle', { class: 'spark-toast-icon' })

    toastMessage.insertAdjacentElement('afterbegin', icon)
  }

  return el
}

function clear(toast, options) {
  if (toast.classList.contains('out')) {
    return
  }

  toast.classList.add('out')

  Event.afterAnimation(toast, function() {
    if (toast.parentNode) {
      toast.parentNode.removeChild(toast)
    }

    if (options.onClose && typeof(options.onClose) == 'function') {
      options.onClose()
    }
  }, true)
}

var Toast = {
  new: function(options) {
    var options = options || {}

    if (!options.message) {
      throw new Error('You need a `message` to use a Toast')
    }

    if (!options.theme) {
      throw new Error('You need a `theme` to use Toast')
    }

    var themes = ['success', 'danger', 'info']
    if (!themes.includes(options.theme)) {
      throw new Error(
        'You need a valid `theme` to use Toast: `success`, `danger`, or `info`'
      )
    }

    var timeout = 6000
    if (typeof(options.timeout) == 'boolean') {
      if (String(options.timeout) == 'true') {
        timeout = 6000
      } else {
        timeout = false
      }
    } else if (options.timeout && typeof(options.timeout) == 'number') {
      timeout = options.timeout
    }

    var containerEl = document.querySelector('.spark-toast-container')
    if (!containerEl) {
      containerEl = createContainer()
    }

    var toast = createToast(options.message, options.theme)
    containerEl.appendChild(toast)

    Event.one(toast, 'click', '[data-dismiss]', function() {
      clear(toast, options)
    })

    if (timeout) {
      Event.delay(clear, timeout, toast, options)
    }
  }
}

module.exports = {
  new: Toast.new
}
