module Spark
  class Block < Spark::Component::Base
    GUTTER = %i[small base medium large xl].freeze
    LAYOUT = %i[flush padded].freeze
    SPACER_SIZE = %i[base].freeze

    attribute gutter: :small, action_gutter: :small, layout: :flush, wrap: true

    validates_attr :gutter, choices: GUTTER
    validates_attr :action_gutter, choices: GUTTER
    validates_attr :layout, choices: [:padded, :flush]

    element :item, multiple: true do
      attribute :expand

      def initialize(*)
        super

        classname.add("expand") if @expand
      end
    end

    element :spacer do
      attribute :size
      validates_attr :size, choices: SPACER_SIZE

      def initialize(*)
        super

        # This is an attempt to determine if a spacer should be added to the list of items, or actions.
        # If no actions have been added yet, add the spacer to the items.
        if _parent.actions.empty?
          _parent.items << self
        else
          _parent.actions << self
        end

        classname.add('block-spacer')
        classname.add("size-#{@size}")
      end
    end

    element :action, multiple: true do
      attribute :expand

      def initialize(*)
        super

        classname.add(:expand) if @expand
      end
    end

    def initialize(*)
      super

      classname.base = "spark-block"
      classname.add("gutter-#{@gutter}")
      classname.add("action-gutter-#{@action_gutter}")
      classname.add("flex-no-wrap") unless @wrap
      classname.add("layout-#{@layout}")
      tag_attrs.add(role: @role)
    end

    # When direct content is passed but no item or action elements are used
    def content_only?
      items.empty? && actions.empty? && @content.present?
    end
  end
end
