module Spark
  class Button < Spark::Component::Base

    COLORS = %i[blue green yellow red purple]
    SIZES  = %i[xs small base medium large]
    STYLES  = %i[fill outline text]
    THEME  = %i[add danger primary launch]
    TYPE   = %i[button submit reset]

    attribute :text, :href, :hidden, :icon, :icon_only, :value,
              :disabled, :name, :target, type: :button
    data_attribute :color, :size, :style, :theme

    attribute_default_group(theme: {
      primary: {
        color: :blue,
        style: :fill
      },
      add: {
        color: :green,
        icon: :plus,
        style: :fill
      },
      danger: {
        color: :red,
        style: :fill
      },
      launch: {
        style: :text,
        icon: :launch,
        target: "_blank"
      }
    })

    validates_attr :color, choices: COLORS, allow_nil: true
    validates_attr :size, choices: SIZES, allow_nil: true
    validates_attr :style, choices: STYLES, allow_nil: true
    validates_attr :theme, choices: THEME, allow_nil: true
    validates_attr :type, choices: TYPE
    validates_attr :text, presence: true, if: -> { @content.nil? }

    # Icon button validation
    validates_attr :icon, presence: true, if: -> { @icon_only }

    def initialize(*)
      super
      tag_attrs.add attr_hash(:value, :disabled, :name, :target)

      if (@href && (data[:remote] || data[:method]))
        tag_attrs.add rel: "nofollow"
      end

      if @icon_only
        aria.add label: @text
        classname.add("icon-only")
      end

      aria.add attr_hash(:hidden)

      tag_attrs.add(type: :button)
      classname.base = "spark-button"
    end
  end
end
