module Spark
  class Checklist < Spark::Component::Base
    attribute :collapse_after

    validates_attr :collapse_after, numericality: { only_integer: true }, allow_blank: true

    element :item, multiple: true, component: Spark::Checklist::Item
    element :complete_content

    def initialize(*)
      super
      data.add checklist_id: unique_id

      classname.base = 'spark-checklist-container'
    end

    def sort_items
      items.sort_by! { |item| item.attribute_completed ? 1 : 0 }
      add_collapse_item_class if @collapse_after
      items
    end

    private

    def complete?
      completed_items.length == items.length
    end

    def completed_items
      @completed_items ||= items.select do |item| 
        item.attribute_completed == true
      end
    end

    def can_collapse?
      @collapse_after && @collapse_after < items.length
    end

    def add_collapse_item_class
      items.slice(@collapse_after, items.size).each do |item|
        item.classname.add("collapse-item")
      end
    end

    def button_toggle
      %(collapsed; #{selector(:data, :checklist_id)})
    end
  end
end
