module Embed
  class WistiaMedia < Spark::Component::Base
    attribute :video_id, :background_image_url, :title, :subtitle, :height, :width

    validates_attr :video_id, presence: true
    validates_attr :background_image_url, presence: true
    validates_attr :height, presence: true, numericality: { only_integer: true }
    validates_attr :width, numericality: { only_integer: true }, allow_blank: true

    def initialize(*)
      super

      classname.base = 'spark-wistia-media'
      wistia_embed_options = {
        class: "wistia_embed wistia_async_#{video_id} popover=true seo=false popoverContent=link popoverAnimation=none",
        style: 'height:100%;width:100%'
      }

      wistia_embed_background_options = {
        class: 'wistia-embed-background',
        style: "background-image: url('#{background_image_url}')"
      }

      unless height.blank?
        tag_attrs.merge!(style: "height: #{height}px;max-width: #{width}px")
      end
    end
  end
end
