module Spark
  class Grid < Spark::Component::Base
    COLUMN = [1,2,3,4,6,12].freeze
    GUTTER = %i[small base medium large xl].freeze
    ALIGN  = %i[left center right split].freeze

    attribute :span, :columns, :gutter_match, :reverse, wrap: true
    data_attribute :gutter, :grid_min, :column_min, align: :left

    validates_attr :span, choices: Spark::Grid::Column::SPAN, allow_nil: true
    validates_attr :align, choices: Spark::Grid::ALIGN
    validates_attr :columns, choices: COLUMN, allow_nil: true
    validates_attr :gutter, choices: GUTTER, allow_nil: true, unless: -> { @gutter == false }
    validates_attr :column_min, numericality: { only_integer: true }, allow_nil: true
    validates_attr :grid_min, numericality: { only_integer: true }, allow_nil: true

    element :column, multiple: true, component: Spark::Grid::Column

    def initialize(*)
      super

      raise "Grids do not support both cols and span. Set `span` (default span for columns), or `cols` (the number of columns per row)" if @span && @columns
      @gutter ||= :base if @gutter_match
      @span = 12 / @columns if @columns

      classname.base = "spark-grid"
      classname.add("gutter-match") if (@gutter && @gutter_match) || @column_min || @grid_min
      classname.add("flex-no-wrap") unless @wrap
      classname.add("reverse") if @reverse

      set_element_attribute_default(:column, span: @span)
    end
  end
end
