module Spark
  class Grid
    class Column < Spark::Component::Base
      SPAN_NAME = { half: 6, third: 4, fourth: 3, sixth: 2 }.freeze
      SPAN      = [*(1..12), *SPAN_NAME.keys, :auto].freeze
      OFFSET    = [*(1..11), *SPAN_NAME.keys].freeze

      attribute :expand_content
      data_attribute :span, :offset

      validates_attr :span, choices: SPAN, allow_nil: true
      validates_attr :offset, choices: OFFSET, allow_nil: true

      def initialize(*)
        super
        classname.base = "spark-grid-column"
        classname.add("expand-content") if @expand_content
      end

      def span_name(size)
        return if size.nil?
        if size.is_a? Integer
          SPAN[size - 1]
        elsif size.is_a? Float
          SPAN[(SPAN.size * size) - 1]
        elsif size = SPAN_NAME[size.to_sym]
          SPAN[(12 / size) - 1]
        end
      end
    end
  end
end
