module Spark
  module Nav
    class Sidebar < Spark::Component::Base
      THEME = %i[default primary]
      attribute :label, toggle: true
      attribute theme: :default
      validates_attr :label, presence: true
      validates_attr :theme, choices: THEME

      element :header
      element :footer
      element :group, component: Spark::Nav::Group, multiple: true do
        attribute :title
      end

      element :tree, component: Spark::Nav::Tree, multiple: true do
        attribute :title

        def initialize(*)
          super
          # Add self to items to ensure groups are rendered
          # in the correct order as items enumerate
          _parent.groups << self
          classname.add("spark-nav-group")
        end
      end

      def initialize(*)
        super

        classname.base = "spark-nav-sidebar"
        classname.add("theme-#{@theme}")
        data.add(nav_id: unique_id)

        if root_sidebar?
          data.add(scroll: true) 
          @toggle = false
        end

        aria.add(label: @label)
      end

      private

      def root_sidebar?
        @id && @id.to_sym == :sidebar
      end

      # Tries to find the text for the active nav item, otherwise shows sidebar label text
      def sidebar_button_text
        # Find any navs which have active items
        active_nav = groups.find { |group|
          group.active_item
        }

        # Use the active item text or the label for the sidebar
        active_nav&.active_item&.attribute_text || @label
      end
    end
  end
end
