module Spark
  module Nav
    class Tree < Spark::Component::Base
      attribute :id, :label, :group_icon_size, role: :tree, icon_size: :medium

      element :item, multiple: true, component: Spark::Nav::Item
      element :group, component: Spark::Nav::Tree::Group do
        def initialize(*)
          super
          # Add self to items to ensure groups are rendered
          # in the correct order as items enumerate
          _parent.items << self
          set_element_attribute_default(:item, icon_size: @icon_size)
        end

        # Add cookie detection to expansion test
        def expanded?
          _parent.group_expanded?(node) || super
        end
      end

      def initialize(*)
        super

        classname.base = "nav-tree"
        data.add(tree: @id)
        aria.add(label: @label)
        tag_attrs.add(role: :tree)

        # Group icon should inherit default size
        set_element_attribute_default(:item, icon_size: @icon_size)
        set_element_attribute_default(:group, icon_size: @group_icon_size || @icon_size)
      end

      def group_expanded?(node)
        @_cookie ||= begin
          view_context.parse_cookie( @id )
        rescue
          nil
        end

        @_cookie && !!@_cookie[ node ]
      end

      def active_item
        item = items.find(&:active?)
        # If the item is a group check return its active_item
        item = item.active_item if item.respond_to?(:active_item)
        item
      end
    end
  end
end
