module Spark
  module ModalHelper 
    class ModalPanel < Spark::Helper

      def initialize( options = {} )
        @options = add_class( options, 'modal-panel' )
        @options = Spark::Helpers.set_aria_keys( @options )
        @options = ({ 'aria-modal'=> true, 'aria-hidden'=> true }).merge(@options)
      end

      def display( body )
        if first_header = body.scan(/\A<header.+?\/header>/m).first
          body = body.sub( first_header, '' ).html_safe
          first_header = first_header.html_safe
        end

        if last_footer = body.scan(/<footer.+?\/footer>\z/m).first
          body = body.sub(last_footer, '').html_safe
          last_footer = last_footer.html_safe
        end

        content_tag( :section, @options ) do
          concat first_header
          concat content_tag( :div, class: 'modal-panel-content' ) {
            concat body
          }
          concat last_footer
        end
      end

      def header( text = nil, options = {}, &block )
        options, text = text, nil if text.is_a? Hash

        options = add_class( options, "modal-panel-header" )

        content_tag( :header, options ) {
          concat content_tag( :h4, text, class: 'modal-panel-title' ) if text
          concat capture( &block ).html_safe if block_given?
        }
      end

      def footer( options = {}, &block )
        options = add_class options, "modal-panel-footer"

        content_tag( :footer, options ) {
          concat capture( &block ).html_safe if block_given?
        }
      end
    end
  end
end
