module Spark
  class Helper < SparkEngine::BlockHelper
  end

  module Helpers
    extend self

    # Move top level keys under data:
    def set_data_keys( options={}, tl_keys=[] )
      tl_keys = tl_keys.split(' ') if tl_keys.is_a? String
      tl_keys = tl_keys | DEFAULT_DATA_KEYS
      sub_key options, tl_keys, 'data'
    end

    # Convert aria: { key: val } to 'aria-key' => val
    def set_aria_keys( options, tl_keys=[] )

      # Add 'aria' parent to passed top level keys
      tl_keys = tl_keys.split(' ') if tl_keys.is_a? String
      tl_keys = tl_keys | DEFAULT_ARIA_KEYS

      options = sub_key options, tl_keys, 'aria'
      opts = {}

      options['aria'].each do |key, val|
        opts["aria-#{key}"] = val
      end

      options.delete 'aria'
      options.merge! opts

      options
    end

    # Move top level keys under new parent key
    def sub_key(options, tl_keys, parent)
      options = dasherize_keys( options )
      tl_keys = tl_keys.split(' ') if tl_keys.is_a? String

      parent = parent.to_s if parent.is_a?( Symbol ) 
      options[parent] ||= {}

      new_hash = {}

      # Look through option keys and reparent keys which match dasherized search keys
      options.each do |k,v|
        new_hash[dasherize k] = options.delete k if tl_keys.include? dasherize k
      end

      options[parent].merge! new_hash
      options

    end

    def set_type_defaults( type, options={} )
      ( INPUT_OPTIONS[type] || {} ).deep_merge options
    end

    def dasherize_keys(obj)
      obj.deep_transform_keys{ |key| dasherize(key.to_s) }
    end

    def dasherize(input)
      input.strip.gsub(/[\W,_]+/, '-')
    end

  end

  DEFAULT_DATA_KEYS = %w(toggle show hide show-menu show-panel show-dialog stack stack-nav)
  DEFAULT_ARIA_KEYS = %w(haspopup expanded)

  INPUT_OPTIONS = {
    email: {
      type: "email",
      placeholder: "Email address",
      pattern: "[^@]+@[^@]+\\.[a-zA-Z]{2,}",
      autocorrect: "off",
      autocapitalize: "off",
      spellcheck: "false",
      data: { message: "Please enter a valid email address." }
    },

    password: {
      type: "password",
      placeholder: "Password"
    },

    text: {
      type: "text"
    },

    search: {
      autocorrect: "off",
      role: "search",
      type: "text"
    },

    number: {
      type: "number",
      step: "any"
    },

    tel: {
      type: "tel",
      placeholder: "Phone number"
    },

    url: {
      type: "text",
      placeholder: "Web address",
      autocorrect: "off",
      autocapitalize: "off",
      spellcheck: "false",
      pattern: ".+\\.[a-zA-Z]{2,}"
    },

    card_number: {
      type: "text",
      required: true,
      pattern: "[0-9 -]{13,20}",
      placeholder: "Credit Card Number",
      data: {
        stripe: "number",
        message: "Please enter a valid credit card number."
      }
    },

    card_month: {
      type: "text",
      required: true,
      pattern: "0[1-9]|1[012]",
      placeholder: "MM",
      data: {
        stripe: "exp_month",
        message: "Please enter a valid expiration month."
      }
    },

    card_year: {
      type: "text",
      required: true,
      pattern: "[0-9]{4}",
      placeholder: "YYYY",
      data: {
        stripe: "exp_year",
        message: "Please enter a valid expiration year."
      }
    },

    card_cvc: {
      type: "text",
      required: true,
      pattern: "[0-9]{3,4}",
      placeholder: "CVC",
      data: {
        stripe: "cvc",
        message: "Please enter a valid security code."
      }
    }
  }
end
